----------------------------------------------------------------------------------
-- Company: ETH Zurich
-- Engineer: Pascal Engeler <engeler.pascal@gmail.com>
-- 
-- Create Date:    13:54:17 07/04/2024 
-- Design Name: 
-- Module Name:    CommandDecoder - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity CommandDecoder is
    Port ( clk100: in STD_LOGIC;
           data : in  STD_LOGIC_VECTOR (7 downto 0);
           cmd_received : in  STD_LOGIC;
           readback_idle : in  STD_LOGIC;
           sweeper_idle : in  STD_LOGIC;
           readback_start : out  STD_LOGIC;
           sweeper_start : out  STD_LOGIC;
           n_ringup_mem_wr_en : out  STD_LOGIC;
           n_meas_mem_wr_en : out  STD_LOGIC;
           n_sweep_mem_wr_en : out  STD_LOGIC;
           dphi_mem_wr_en : out  STD_LOGIC;
           ddphi_mem_wr_en : out  STD_LOGIC;
           mem_addr : out  STD_LOGIC_VECTOR (9 downto 0);
           reg : out  STD_LOGIC_VECTOR (31 downto 0);
           n_freqs : out  STD_LOGIC_VECTOR (9 downto 0));
end CommandDecoder;



architecture Behavioral of CommandDecoder is

type STATE_T is (IDLE_STATE, CMD_STATE1, CMD_STATE2);

signal current_state: STATE_T := IDLE_STATE;

signal data_sig : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
signal cmd_received_old : STD_LOGIC := '0';
signal reg_sig : STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
signal n_freqs_sig : STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal mem_addr_sig : STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal readback_start_sig : STD_LOGIC := '0';
signal sweeper_start_sig : STD_LOGIC := '0';
signal current_cmd : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
signal dphi_mem_wr_en_sig: STD_LOGIC := '0';
signal n_ringup_mem_wr_en_sig: STD_LOGIC := '0';
signal n_meas_mem_wr_en_sig: STD_LOGIC := '0';
signal n_sweep_mem_wr_en_sig: STD_LOGIC := '0';
signal ddphi_mem_wr_en_sig: STD_LOGIC := '0';

begin

trigger_update: process(clk100)
begin
	if rising_edge(clk100) then
		cmd_received_old <= cmd_received;
	end if;
end process;

update_proc: process(clk100)
begin
	if rising_edge(clk100) then 
	--current_cmd <= current_cmd;
	--current_state <= current_state;
	--reg_sig <= reg_sig;
	--mem_addr_sig <= mem_addr_sig;
	--dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
	--n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
	--n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
	--n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
	--ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
	--n_freqs_sig <= n_freqs_sig;
	--sweeper_start_sig <= sweeper_start_sig;
	--readback_start_sig <= readback_start_sig;
		if (readback_idle = '1' AND sweeper_idle  = '1') then
			case current_state is
				when IDLE_STATE =>
					reg_sig <= reg_sig;
					mem_addr_sig <= mem_addr_sig;
					dphi_mem_wr_en_sig <= '0';
					n_meas_mem_wr_en_sig <= '0';
					n_ringup_mem_wr_en_sig <= '0';
					n_sweep_mem_wr_en_sig <= '0';
					ddphi_mem_wr_en_sig <= '0';
					n_freqs_sig <= n_freqs_sig;
					sweeper_start_sig <= '0';
					readback_start_sig <= '0';
					if (cmd_received_old = '0') AND (cmd_received = '1') then
						current_cmd <= data_sig;
						current_state <= CMD_STATE1;
					else
						current_cmd <= current_cmd;
						current_state <= current_state;
					end if;
				when CMD_STATE1 =>
					if current_cmd(7 downto 5) = "001" then
						reg_sig(4 downto 0) <= current_cmd(4 downto 0);
						reg_sig(31 downto 5) <= reg_sig(31 downto 5);
						current_state <= IDLE_STATE;
						--still signals
						current_cmd <= current_cmd;
						mem_addr_sig <= mem_addr_sig;
						dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
						n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
						n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
						n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
						ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
						n_freqs_sig <= n_freqs_sig;
						sweeper_start_sig <= sweeper_start_sig;
						readback_start_sig <= readback_start_sig;
					elsif current_cmd(7 downto 5) = "010" then
						reg_sig(9 downto 5) <= current_cmd(4 downto 0);
						reg_sig(31 downto 10) <= reg_sig(31 downto 10);
						reg_sig(4 downto 0) <= reg_sig(4 downto 0);
						current_state <= IDLE_STATE;
						--still signals
						current_cmd <= current_cmd;
						mem_addr_sig <= mem_addr_sig;
						dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
						n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
						n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
						n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
						ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
						n_freqs_sig <= n_freqs_sig;
						sweeper_start_sig <= sweeper_start_sig;
						readback_start_sig <= readback_start_sig;
					elsif current_cmd(7 downto 5) = "011" then
						reg_sig(14 downto 10) <= current_cmd(4 downto 0);
						reg_sig(31 downto 15) <= reg_sig(31 downto 15);
						reg_sig(9 downto 0) <= reg_sig(9 downto 0);
						current_state <= IDLE_STATE;
						--still signals
						current_cmd <= current_cmd;
						mem_addr_sig <= mem_addr_sig;
						dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
						n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
						n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
						n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
						ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
						n_freqs_sig <= n_freqs_sig;
						sweeper_start_sig <= sweeper_start_sig;
						readback_start_sig <= readback_start_sig;
					elsif current_cmd(7 downto 5) = "100" then
						reg_sig(19 downto 15) <= current_cmd(4 downto 0);
						reg_sig(31 downto 20) <= reg_sig(31 downto 20);
						reg_sig(14 downto 0) <= reg_sig(14 downto 0);
						current_state <= IDLE_STATE;
						--still signals
						current_cmd <= current_cmd;
						mem_addr_sig <= mem_addr_sig;
						dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
						n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
						n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
						n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
						ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
						n_freqs_sig <= n_freqs_sig;
						sweeper_start_sig <= sweeper_start_sig;
						readback_start_sig <= readback_start_sig;
					elsif current_cmd(7 downto 5) = "101" then
						reg_sig(24 downto 20) <= current_cmd(4 downto 0);
						reg_sig(31 downto 25) <= reg_sig(31 downto 25);
						reg_sig(19 downto 0) <= reg_sig(19 downto 0);
						current_state <= IDLE_STATE;
						--still signals
						current_cmd <= current_cmd;
						mem_addr_sig <= mem_addr_sig;
						dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
						n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
						n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
						n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
						ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
						n_freqs_sig <= n_freqs_sig;
						sweeper_start_sig <= sweeper_start_sig;
						readback_start_sig <= readback_start_sig;
					elsif current_cmd(7 downto 5) = "110" then
						reg_sig(29 downto 25) <= current_cmd(4 downto 0);
						reg_sig(31 downto 30) <= reg_sig(31 downto 30);
						reg_sig(24 downto 0) <= reg_sig(24 downto 0);
						current_state <= IDLE_STATE;
						--still signals
						current_cmd <= current_cmd;
						mem_addr_sig <= mem_addr_sig;
						dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
						n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
						n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
						n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
						ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
						n_freqs_sig <= n_freqs_sig;
						sweeper_start_sig <= sweeper_start_sig;
						readback_start_sig <= readback_start_sig;
					elsif current_cmd(7 downto 5) = "111" then
						reg_sig(31 downto 30) <= current_cmd(1 downto 0);
						reg_sig(29 downto 0) <= reg_sig(29 downto 0);
						current_state <= IDLE_STATE;
						--still signals
						current_cmd <= current_cmd;
						mem_addr_sig <= mem_addr_sig;
						dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
						n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
						n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
						n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
						ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
						n_freqs_sig <= n_freqs_sig;
						sweeper_start_sig <= sweeper_start_sig;
						readback_start_sig <= readback_start_sig;
					else --current_cmd(7 downto 5) = "000"
						if current_cmd(4 downto 0) = "00000" then
							mem_addr_sig(9 downto 0) <= reg_sig(9 downto 0);
							current_state <= IDLE_STATE;
							--still signals
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
							n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
							n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
							n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
							ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
							n_freqs_sig <= n_freqs_sig;
							sweeper_start_sig <= sweeper_start_sig;
							readback_start_sig <= readback_start_sig;
						elsif current_cmd(4 downto 0) = "00001" then
							dphi_mem_wr_en_sig <= '1';
							current_state <= CMD_STATE2;
							--still signals
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							mem_addr_sig <= mem_addr_sig;
							n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
							n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
							n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
							ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
							n_freqs_sig <= n_freqs_sig;
							sweeper_start_sig <= sweeper_start_sig;
							readback_start_sig <= readback_start_sig;
						elsif current_cmd(4 downto 0) = "00010" then
							n_meas_mem_wr_en_sig <= '1';
							current_state <= CMD_STATE2;
							--still signals
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							mem_addr_sig <= mem_addr_sig;
							dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
							n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
							n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
							ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
							n_freqs_sig <= n_freqs_sig;
							sweeper_start_sig <= sweeper_start_sig;
							readback_start_sig <= readback_start_sig;
						elsif current_cmd(4 downto 0) = "00011" then
							n_ringup_mem_wr_en_sig <= '1';
							current_state <= CMD_STATE2;
							--still signals
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							mem_addr_sig <= mem_addr_sig;
							dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
							n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
							n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
							ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
							n_freqs_sig <= n_freqs_sig;
							sweeper_start_sig <= sweeper_start_sig;
							readback_start_sig <= readback_start_sig;
						elsif current_cmd(4 downto 0) = "00100" then
							n_sweep_mem_wr_en_sig <= '1';
							current_state <= CMD_STATE2;
							--still signals
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							mem_addr_sig <= mem_addr_sig;
							dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
							n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
							n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
							ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
							n_freqs_sig <= n_freqs_sig;
							sweeper_start_sig <= sweeper_start_sig;
							readback_start_sig <= readback_start_sig;
						elsif current_cmd(4 downto 0) = "00101" then
							ddphi_mem_wr_en_sig <= '1';
							current_state <= CMD_STATE2;
							--still signals
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							mem_addr_sig <= mem_addr_sig;
							dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
							n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
							n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
							n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
							n_freqs_sig <= n_freqs_sig;
							sweeper_start_sig <= sweeper_start_sig;
							readback_start_sig <= readback_start_sig;
						elsif current_cmd(4 downto 0) = "00111" then
							n_freqs_sig(9 downto 0) <= reg_sig(9 downto 0);
							current_state <= IDLE_STATE;
							--still signals
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							mem_addr_sig <= mem_addr_sig;
							dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
							n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
							n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
							n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
							ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
							sweeper_start_sig <= sweeper_start_sig;
							readback_start_sig <= readback_start_sig;
						elsif current_cmd(4 downto 0) = "01000" then
							sweeper_start_sig <= '1';
							current_state <= CMD_STATE2;
							--still signals
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							mem_addr_sig <= mem_addr_sig;
							dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
							n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
							n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
							n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
							ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
							n_freqs_sig <= n_freqs_sig;
							readback_start_sig <= readback_start_sig;
						elsif current_cmd(4 downto 0) = "01001" then
							readback_start_sig <= '1';
							current_state <= CMD_STATE2;
							--still signals
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							mem_addr_sig <= mem_addr_sig;
							dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
							n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
							n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
							n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
							ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
							n_freqs_sig <= n_freqs_sig;
							sweeper_start_sig <= sweeper_start_sig;
						else
							current_state <= IDLE_STATE;
							current_cmd <= current_cmd;
							reg_sig <= reg_sig;
							mem_addr_sig <= mem_addr_sig;
							dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
							n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
							n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
							n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
							ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
							n_freqs_sig <= n_freqs_sig;
							sweeper_start_sig <= sweeper_start_sig;
							readback_start_sig <= readback_start_sig;
						end if;
					end if;
				when CMD_STATE2 =>
					current_state <= IDLE_STATE;
					current_cmd <= current_cmd;
					reg_sig <= reg_sig;
					mem_addr_sig <= mem_addr_sig;
					dphi_mem_wr_en_sig <= '0';
					n_meas_mem_wr_en_sig <= '0';
					n_ringup_mem_wr_en_sig <= '0';
					n_sweep_mem_wr_en_sig <= '0';
					ddphi_mem_wr_en_sig <= '0';
					n_freqs_sig <= n_freqs_sig;
					sweeper_start_sig <= '0';
					readback_start_sig <= '0';
				
			end case;
			else 
				--command ignored, device not idle
				--todo: beep here or turn on an led or something
				current_cmd <= current_cmd;
				current_state <= current_state;
				reg_sig <= reg_sig;
				mem_addr_sig <= mem_addr_sig;
				dphi_mem_wr_en_sig <= dphi_mem_wr_en_sig;
				n_meas_mem_wr_en_sig <= n_meas_mem_wr_en_sig;
				n_ringup_mem_wr_en_sig <= n_ringup_mem_wr_en_sig;
				n_sweep_mem_wr_en_sig <= n_sweep_mem_wr_en_sig;
				ddphi_mem_wr_en_sig <= ddphi_mem_wr_en_sig;
				n_freqs_sig <= n_freqs_sig;
				sweeper_start_sig <= sweeper_start_sig;
				readback_start_sig <= readback_start_sig;
		end if;
	end if;
end process;

reg <= reg_sig;
data_sig <= data;
n_freqs <= n_freqs_sig;
mem_addr <= mem_addr_sig;
readback_start <= readback_start_sig;
sweeper_start <= sweeper_start_sig;
dphi_mem_wr_en <= dphi_mem_wr_en_sig;
n_ringup_mem_wr_en <= n_ringup_mem_wr_en_sig;
n_meas_mem_wr_en <= n_meas_mem_wr_en_sig;
n_sweep_mem_wr_en <= n_sweep_mem_wr_en_sig;
ddphi_mem_wr_en <= ddphi_mem_wr_en_sig;

end Behavioral;