----------------------------------------------------------------------------------
-- Company: ETH Zurich
-- Engineer: Pascal Engeler <engeler.pascal@gmail.com>
-- 
-- Create Date:    21:28:38 07/04/2024 
-- Design Name: 
-- Module Name:    ReadbackControl - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity ReadbackControl is
    Port ( clk100: in STD_LOGIC;
	        start : in  STD_LOGIC;
           end_addr : in  STD_LOGIC_VECTOR (9 downto 0);
           serial_trans_idle : in  STD_LOGIC;
           addr_rd : out  STD_LOGIC_VECTOR (9 downto 0);
           idle : out  STD_LOGIC;
           send : out  STD_LOGIC);
end ReadbackControl;

architecture Behavioral of ReadbackControl is

type STATE_T is (IDLE_S, APP_ADDR, SEND_UP, SEND_DN, WAIT1, INC_ADDR, WAIT2);

signal current_state: STATE_T := IDLE_S;

signal send_sig: STD_LOGIC := '0';
signal addr_sig: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal tx_idle_sig: STD_LOGIC;
signal end_addr_sig: STD_LOGIC_VECTOR(9 downto 0);
signal start_old_sig: STD_LOGIC := '0';
signal idle_sig: STD_LOGIC := '1';


begin

start_proc: process(clk100)
begin
	if rising_edge(clk100) then
		start_old_sig <= start;
	end if;
end process;


update_proc: process(clk100)
begin
	if rising_edge(clk100) then
		case current_state is
		when IDLE_S =>
			if start_old_sig = '0' and start = '1' then
				current_state <= APP_ADDR;
				idle_sig <= '0';
			else
				current_state <= current_state;
				idle_sig <= '1';
			end if;
			send_sig <= '0';
			addr_sig <= (others => '0');
		when APP_ADDR =>
			current_state <= SEND_UP;
			send_sig <= '0';
			addr_sig <= addr_sig;
			idle_sig <= '0';
		when SEND_UP =>
			current_state <= SEND_DN;
			send_sig <= '1';
			addr_sig <= addr_sig;
			idle_sig <= '0';
		when SEND_DN =>
		--wait for  transmission to start
			if tx_idle_sig = '0' then
				current_state <= WAIT1;
			else 
				current_state <= SEND_DN;
			end if;
			send_sig <= '0';
			addr_sig <= addr_sig;
			idle_sig <= '0';
		when WAIT1 =>
			if tx_idle_sig = '1' then
				current_state <= INC_ADDR;
			else
				current_state <= WAIT1;
			end if;
			send_sig <= '0';
			addr_sig <= addr_sig;
			idle_sig <= '0';
		when INC_ADDR =>
			current_state <= WAIT2;
			send_sig <= '0';
			addr_sig <= addr_sig + 1;
			idle_sig <= '0';
		when WAIT2 =>
			if addr_sig = end_addr_sig then
				current_state <= IDLE_S;
			else
				current_state <= APP_ADDR;
			end if;
			send_sig <= '0';
			addr_sig <= addr_sig;
			idle_sig <= '0';
		
		end case;
	end if;
end process;

tx_idle_sig <= serial_trans_idle;
end_addr_sig <= end_addr;
addr_rd <= addr_sig;
send <= send_sig;
idle <= idle_sig;

end Behavioral;

